@@ -11,9 +11,8 @@ from argparse import ArgumentParser, Namespace
 
 import mistune
 from docutils import io, nodes, statemachine, utils
-from docutils.core import ErrorString
 from docutils.parsers import rst
-from docutils.utils import SafeString, column_width
+from docutils.utils import column_width
 from pkg_resources import get_distribution
 
 __version__ = get_distribution("m2r2").version
@@ -81,9 +80,18 @@ def parse_options():
 
 
 class RestBlockGrammar(mistune.BlockGrammar):
-    directive = re.compile(r"^( *\.\..*?)\n(?=\S)", re.DOTALL | re.MULTILINE,)
-    oneline_directive = re.compile(r"^( *\.\..*?)$", re.DOTALL | re.MULTILINE,)
-    rest_code_block = re.compile(r"^::\s*$", re.DOTALL | re.MULTILINE,)
+    directive = re.compile(
+        r"^( *\.\..*?)\n(?=\S)",
+        re.DOTALL | re.MULTILINE,
+    )
+    oneline_directive = re.compile(
+        r"^( *\.\..*?)$",
+        re.DOTALL | re.MULTILINE,
+    )
+    rest_code_block = re.compile(
+        r"^::\s*$",
+        re.DOTALL | re.MULTILINE,
+    )
 
 
 class RestBlockLexer(mistune.BlockLexer):
@@ -206,6 +214,7 @@ class RestRenderer(mistune.Renderer):
     def __init__(self, *args, **kwargs):
         self.parse_relative_links = kwargs.pop("parse_relative_links", False)
         self.anonymous_references = kwargs.pop("anonymous_references", False)
+        self.use_mermaid = kwargs.pop("use_mermaid", False)
         super(RestRenderer, self).__init__(*args, **kwargs)
         if not _is_sphinx:
             parse_options()
@@ -226,6 +235,8 @@ class RestRenderer(mistune.Renderer):
     def block_code(self, code, lang=None):
         if lang == "math":
             first_line = "\n.. math::\n\n"
+        elif lang == "mermaid" and self.use_mermaid:
+            first_line = "\n.. mermaid::\n\n"
         elif lang:
             first_line = "\n.. code-block:: {}\n\n".format(lang)
         elif _is_sphinx:
@@ -564,6 +575,7 @@ class M2RParser(rst.Parser, object):
             parse_relative_links=config.m2r_parse_relative_links,
             anonymous_references=config.m2r_anonymous_references,
             disable_inline_math=config.m2r_disable_inline_math,
+            use_mermaid=config.m2r_use_mermaid,
         )
         super(M2RParser, self).parse(converter(inputstring), document)
 
@@ -617,12 +629,12 @@ class MdInclude(rst.Directive):
             raise self.severe(
                 'Problems with "%s" directive path:\n'
                 'Cannot encode input file path "%s" '
-                "(wrong locale?)." % (self.name, SafeString(path))
+                "(wrong locale?)." % (self.name, str(path))
             )
         except IOError as error:
             raise self.severe(
                 'Problems with "%s" directive path:\n%s.'
-                % (self.name, ErrorString(error))
+                % (self.name, io.error_string(error))
             )
 
         # read from the file
@@ -636,7 +648,7 @@ class MdInclude(rst.Directive):
                 rawtext = include_file.read()
         except UnicodeError as error:
             raise self.severe(
-                'Problem with "%s" directive:\n%s' % (self.name, ErrorString(error))
+                'Problem with "%s" directive:\n%s' % (self.name, io.error_string(error))
             )
 
         config = self.state.document.settings.env.config
@@ -645,6 +657,7 @@ class MdInclude(rst.Directive):
             parse_relative_links=config.m2r_parse_relative_links,
             anonymous_references=config.m2r_anonymous_references,
             disable_inline_math=config.m2r_disable_inline_math,
+            use_mermaid=config.m2r_use_mermaid,
         )
         include_lines = statemachine.string2lines(
             converter(rawtext), tab_width, convert_whitespace=True
@@ -661,6 +674,9 @@ def setup(app):
     app.add_config_value("m2r_parse_relative_links", False, "env")
     app.add_config_value("m2r_anonymous_references", False, "env")
     app.add_config_value("m2r_disable_inline_math", False, "env")
+    app.add_config_value(
+        "m2r_use_mermaid", "sphinxcontrib.mermaid" in app.config.extensions, "env"
+    )
     try:
         app.add_source_parser(".md", M2RParser)  # for older sphinx versions
     except (TypeError, AttributeError):
@@ -668,7 +684,9 @@ def setup(app):
         app.add_source_parser(M2RParser)
     app.add_directive("mdinclude", MdInclude)
     metadata = dict(
-        version=__version__, parallel_read_safe=True, parallel_write_safe=True,
+        version=__version__,
+        parallel_read_safe=True,
+        parallel_write_safe=True,
     )
     return metadata
 
